// +build linux darwin netbsd

package utils

import (
	"fmt"
	"syscall"

	"github.com/docker/docker/pkg/mount"
	"golang.org/x/sys/unix"
)

func Major(dev uint64) uint32 {
	return unix.Major(dev)
}

func Minor(dev uint64) uint32 {
	return unix.Minor(dev)
}

func Mkdev(major, minor uint32) uint64 {
	return unix.Mkdev(major, minor)
}

// GetStattFromPath is a helper function that returns the Stat_t
// object for a given path
func GetStattFromPath(path string) (syscall.Stat_t, error) {
	statInfo := syscall.Stat_t{}
	err := syscall.Lstat(path, &statInfo)
	if err != nil {
		return statInfo, err
	}
	return statInfo, nil
}

// GetDeviceNameFromPath iterates through the mounts and matches
// the one that the provided path is on
func GetDeviceNameFromPath(path string) (string, error) {
	statInfo, err := GetStattFromPath(path)
	if err != nil {
		return "", err
	}

	mounts, err := mount.GetMounts()
	if err != nil {
		return "", err
	}

	queryMajor := int(Major(uint64(statInfo.Dev)))
	queryMinor := int(Minor(uint64(statInfo.Dev)))

	for _, mount := range mounts {
		if mount.Minor == queryMinor && mount.Major == queryMajor {
			return mount.Source, nil
		}
	}

	return "", fmt.Errorf("no match found")
}
